@@ -1,24 +1,17 @@
-# -*- coding: utf-8 -*-
-"""
-flask.logging
-~~~~~~~~~~~~~
-
-:copyright: 2010 Pallets
-:license: BSD-3-Clause
-"""
-from __future__ import absolute_import
-
 import logging
 import sys
-import warnings
+import typing as t
 
 from werkzeug.local import LocalProxy
 
 from .globals import request
 
+if t.TYPE_CHECKING:
+    from .app import Flask
+
 
 @LocalProxy
-def wsgi_errors_stream():
+def wsgi_errors_stream() -> t.TextIO:
     """Find the most appropriate error stream for the application. If a request
     is active, log to ``wsgi.errors``, otherwise use ``sys.stderr``.
 
@@ -30,7 +23,7 @@ def wsgi_errors_stream():
     return request.environ["wsgi.errors"] if request else sys.stderr
 
 
-def has_level_handler(logger):
+def has_level_handler(logger: logging.Logger) -> bool:
     """Check if there is a handler in the logging chain that will handle the
     given logger's :meth:`effective level <~logging.Logger.getEffectiveLevel>`.
     """
@@ -44,35 +37,21 @@ def has_level_handler(logger):
         if not current.propagate:
             break
 
-        current = current.parent
+        current = current.parent  # type: ignore
 
     return False
 
 
 #: Log messages to :func:`~flask.logging.wsgi_errors_stream` with the format
 #: ``[%(asctime)s] %(levelname)s in %(module)s: %(message)s``.
-default_handler = logging.StreamHandler(wsgi_errors_stream)
+default_handler = logging.StreamHandler(wsgi_errors_stream)  # type: ignore
 default_handler.setFormatter(
     logging.Formatter("[%(asctime)s] %(levelname)s in %(module)s: %(message)s")
 )
 
 
-def _has_config(logger):
-    """Decide if a logger has direct configuration applied by checking
-    its properties against the defaults.
-
-    :param logger: The :class:`~logging.Logger` to inspect.
-    """
-    return (
-        logger.level != logging.NOTSET
-        or logger.handlers
-        or logger.filters
-        or not logger.propagate
-    )
-
-
-def create_logger(app):
-    """Get the the Flask apps's logger and configure it if needed.
+def create_logger(app: "Flask") -> logging.Logger:
+    """Get the Flask app's logger and configure it if needed.
 
     The logger name will be the same as
     :attr:`app.import_name <flask.Flask.name>`.
@@ -86,20 +65,6 @@ def create_logger(app):
     """
     logger = logging.getLogger(app.name)
 
-    # 1.1.0 changes name of logger, warn if config is detected for old
-    # name and not new name
-    for old_name in ("flask.app", "flask"):
-        old_logger = logging.getLogger(old_name)
-
-        if _has_config(old_logger) and not _has_config(logger):
-            warnings.warn(
-                "'app.logger' is named '{name}' for this application,"
-                " but configuration was found for '{old_name}', which"
-                " no longer has an effect. The logging configuration"
-                " should be moved to '{name}'.".format(name=app.name, old_name=old_name)
-            )
-            break
-
     if app.debug and not logger.level:
         logger.setLevel(logging.DEBUG)
 
